Package: optedr
Title: Calculating Optimal and D-Augmented Designs for Single- and
        Multi-Factor Models
Version: 3.0.1
Authors@R: 
    c(person(given = "Carlos",
           family = "de la Calle-Arroyo",
           role = c("aut", "cre"),
           email = "carlos.calle.arroyo@gmail.com",
           comment = c(ORCID = "0000-0002-5099-888X")),
      person(given = "Jesús",
           family = "López-Fidalgo",
           role = c("aut"),
           comment = c(ORCID = "0000-0001-7502-8188")), 
      person(given = "Licesio J.",
           family = "Rodríguez-Aragón",
           role = c("aut"),
           comment = c(ORCID = "0000-0003-4970-3877")) 
           )
Description: Calculates D-, Ds-, A-, I- and L-optimal designs, weighted combinations of these via a Compound criterion, and KL-optimal designs for model discrimination, for non-linear single- and multi-factor models, via an implementation of the cocktail algorithm (Yu, 2011, <doi:10.1007/s11222-010-9183-2>). Multi-factor models use design variables x1, x2, … with a named-list design space; single-factor models remain backward compatible. Compares designs via their efficiency, augments any design with a controlled efficiency loss, and provides efficient rounding functions to convert approximate designs to exact ones.
License: GPL-3
Encoding: UTF-8
URL: https://github.com/kezrael/optedr,
        https://github.com/Kezrael/optedr
BugReports: https://github.com/Kezrael/optedr/issues
RoxygenNote: 7.3.1
VignetteBuilder: knitr
Suggests: testthat (>= 3.0.0), mockery, knitr, rmarkdown, markdown, DT,
        shinydashboard, shinyalert, plotly, hrbrthemes, shinyjs,
        orthopolynom, magrittr, tidyverse, nleqslv
Imports: ggplot2, purrr, rlang, crayon, cli, shiny, utils
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-07-03 09:58:42 UTC; Carlos
Author: Carlos de la Calle-Arroyo [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-5099-888X>),
  Jesús López-Fidalgo [aut] (ORCID:
    <https://orcid.org/0000-0001-7502-8188>),
  Licesio J. Rodríguez-Aragón [aut] (ORCID:
    <https://orcid.org/0000-0003-4970-3877>)
Maintainer: Carlos de la Calle-Arroyo <carlos.calle.arroyo@gmail.com>
Repository: CRAN
Date/Publication: 2026-07-03 16:00:02 UTC
