# Generated by roxygen2: do not edit by hand

S3method(plot,optdes)
S3method(print,augment_region)
S3method(print,optdes)
S3method(summary,optdes)
export(augment_design)
export(combinatorial_round)
export(design_efficiency)
export(efficient_round)
export(get_augment_region)
export(make_glm_family)
export(make_kl_fun)
export(opt_des)
export(shiny_augment)
export(shiny_optimal)
