---
title: "Presentación"
author: "Carlos de la Calle"
date: "5/10/2020"
output: md_document
---


## Contact

This project has been developed by Carlos de la Calle-Arroyo, to illustrate the results of his joint work with Jesús López-Fidalgo and Licesio Rodríguez-Aragón. If you find any error, have any doubts about the content, result or methodology, or just want to get more detail or give feedback you can write the developer at: carlos.callearroyo@uclm.es. 
