Help
----

The processes and examples shown here are based in the theoretical
results previous to this work. Given a choice for the weights of points
to add, *α*, and desired efficiency, *e**f**f*<sub>*D*</sub> (bounded by
the first and the initial design, *ζ*), one can calculate the points
that satisfy:

$$
eff\_{D} \\geq (1-\\alpha) \\left\\{ 1+\\frac{\\alpha d(x\_{1},\\zeta)}{1-\\alpha} \\right\\}^{1/m}.
$$

In out work, it has been proved that choosing any number of points,
distributing the weight *α* among them as desired, the efficiency is
greater or equal to the choice of *e**f**f*<sub>*D*</sub>.

The points that satisfy that condition for the choices of the user on
each section of the application are what we call the “feasible region”,
from which the users can choose their points with the choice of
efficiency as a minimum for the efficiency of the resulting design.

The optimal designs are calculated through their analytical expressions,
as are the sensitivity functions.
