---
title: "Presentación"
author: "Carlos de la Calle"
date: "5/10/2020"
output: md_document
---


## Help

The processes and examples shown here are based in the theoretical results previous to this work. Given a choice for the weights of points to add, $\alpha$, and desired efficiency, $eff_{D}$ (bounded by the first and the initial design, $\zeta$), one can calculate the points that satisfy:

\[
eff_{D} \geq (1-\alpha) \left\{ 1+\frac{\alpha d(x_{1},\zeta)}{1-\alpha} \right\}^{1/m}.
\]

In out work, it has been proved that choosing any number of points, distributing the weight $\alpha$ among them as desired, the efficiency is greater or equal to the choice of $eff_D$. 

The points that satisfy that condition for the choices of the user on each section of the application are what we call the "feasible region", from which the users can choose their points with the choice of efficiency as a minimum for the efficiency of the resulting design.

The optimal designs are calculated through their analytical expressions, as are the sensitivity functions. 

