% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{findminval}
\alias{findminval}
\title{Find Minimum Value}
\usage{
findminval(sens, design_space, grid.length)
}
\arguments{
\item{sens}{A numeric function to evaluate (scalar for 1D, named numeric vector for multi-factor).}

\item{design_space}{Numeric vector \code{c(min, max)} for single-factor models, or a
named list \code{list(x1 = c(min, max), ...)} for multi-factor models.}

\item{grid.length}{Number of grid points (1D) or LHS samples (multi-factor).}
}
\value{
The value of the minimum
}
\description{
Searches the minimum of a function over a grid on the design space.
}
