% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{plot.optdes}
\alias{plot.optdes}
\title{Plot function for optdes}
\usage{
\method{plot}{optdes}(x, ...)
}
\arguments{
\item{x}{An object of class \code{optdes}.}

\item{...}{Possible extra arguments (currently unused).}
}
\description{
For single-factor models, overlays the support points on the sensitivity function curve.
For two-factor models, shows a heatmap of the sensitivity function with the support
points overlaid and the Equivalence Theorem contour highlighted.
For models with more than two factors, shows a pairwise scatter matrix with one
panel per pair of design variables and point size proportional to weight.
}
\examples{
rri <- opt_des(criterion = "I-Optimality", model = y ~ a * exp(-b / x),
  parameters = c("a", "b"), par_values = c(1, 1500), design_space = c(212, 422),
  reg_int = c(380, 422))
plot(rri)
}
